#!/bin/sh
#
# Ma_Sys.ma Development Linux Debian Packaging Control Script 1.0.1.7,
# Copyright (c) 2013 Ma_Sys.ma.
# For further info send an e-mail to Ma_Sys.ma@web.de.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

MDPC_SCRIPT_DIR="$(dirname "$0")"
. "$MDPC_SCRIPT_DIR/helpers/conf.sh"

# Main-Method. Invoked from file-level. Entry point function.
# $@: All parameters given to the program.
ma_main() {
	case "$1" in
		(-n|new)         ma_new "$@";;
		(-u|update)      ma_pkgctrl update      "$@";;
		(-o|update_only) ma_pkgctrl update_only "$@";;
		(-b|compile)     ma_pkgctrl compile     "$@";;
		# -s Used internally
		(-s|sync)        ma_pkgctrl sync        "$@";;
		(-f|fakeroot-s)  ma_pkgctrl fakeroot-s  "$@";;
		(-w|work)        ma_work;;
		(-v|vm)          ma_vm;;
		(-c|chroot)      ma_chroot "$@";;
		(-h|--help|help) ma_help;;
		# MEC (strictly speaking no longer required with MAEM)
		(-\?)            ma_usage;;
		(ls)             echo RAW && ma_longusage | cut -c 3-;;
		# default
		(*)              ma_help "$@"; return 1;;
	esac
	return $?
}

ma_new() {
	. "$MDPC_SCRIPT_DIR/helpers/new.sh"
	case "$2" in
		(chroot) ma_new_chroot;;
		(repo)   ma_new_repo;;
		(dirs)   ma_new_dirs;;
		(*)      ma_new_section "$2";;
	esac
	return $?
}

ma_pkgctrl() {
	. "$MDPC_SCRIPT_DIR/helpers/pkgctrl.sh"
	if [ "$1" = "fakeroot-s" -o "$1" = "-f" ]
	then
		ma_pkgctrl_compile_fakeroot "$3" "$4"
	else
		ma_pkgctrl_parse_args "$@" && ma_pkgctrl_iterate "$1"
	fi
	return $?
}

ma_work() {
	cd "$MDPC_SECTION_DIR"

	exec tmux \
		new-session -d "vim \"$MDPC_NOTES\"" \; \
		split-window -d "COLUMNS=79 bash" \; \
		split-window -d \
			"echo Hint: Use mdpc vm to enter a virtual machine; \
			COLUMNS=79 bash" \; \
		split-window -d "vifm . /data" \; \
		select-layout tiled \; \
		select-pane -R \; \
		select-pane -D \; \
		attach
}

ma_vm() {
	if ! ping -w 1 -c 1 $MDPC_VIRTUAL_IP
	then
		virtualbox --startvm debian_packaging_vm &
	fi
	while ! ping -w 1 -c 1 $MDPC_VIRTUAL_IP
	do
		sleep 1
	done
	ssh root@$MDPC_VIRTUAL_IP
}

ma_chroot() {
	shift # Omit "chroot" parameter
	if ! [ "$(id -u)" = "0" ]
	then
		exec su -c "$0 chroot $@"
	fi
	arch=
	param=
	if [ "$1" = "admin" -o "$1" = "" ]
	then
		arch=i386
		param=$1
	else
		arch=$1
		param=$2
	fi
	if [ "$param" = "admin" ]
	then
		echo
		echo You might need to mount proc via mount /proc.
		echo Remember to also umount it afterwards.
		echo
		cd "$MDPC_EXTERNAL_DIR/chroots"
		chroot $arch || ma_fail Chroot failed.
	else
		echo
		echo WARNING: Be careful with rm inside the chroot!
		echo
		mount -o bind "$MDPC_SCRIPT_DIR/.." \
			"$MDPC_EXTERNAL_DIR/chroots/$arch/media/masysmadebpkg" \
			|| ma_fail Could not mount own directory inside chroot.
		cd "$MDPC_EXTERNAL_DIR/chroots"
		chroot $arch su linux-fan || echo Chroot failed.
		umount "$MDPC_EXTERNAL_DIR/chroots/$arch/media/masysmadebpkg" \
						|| ma_fail Could not unmount.
	fi
}

ma_fail() {
	echo ERROR: $@
	exit 1
}

# Display usage information
# $@: Parameters which failed to be parsed.
ma_help() {
	# Copyright
	head -n 6 "$0" | tail -n 4 | cut -c3-
	if [ "$1" != "" ]
	then
		echo "Could not parse arguments: $@"
		echo
	fi
	# so ~ $0
	so="$(basename "$0")"
	echo "USAGE $so <action>       <parameters>"
	echo "USAGE $so -n|new         [<section>|chroot|repo|dirs]"
	echo "USAGE $so -u|update      [<section>] [<package>] [<arch>]"
	echo "USAGE $so -o|update_only [<section>] [<package>] [<arch>]"
	echo "USAGE $so -b|compile     [<section>] [<package>] [<arch>]"
	echo "USAGE $so -s|sync        [<section>] [<package>]"
	echo "USAGE $so -w|work"
	echo "USAGE $so -v|vm"
	echo "USAGE $so -c|chroot      [admin]"
	echo "USAGE $so [--]help"
	echo
	echo "Action description"
	ma_longusage
	echo
	echo "Package description"
	echo "  <section>   $MDPC_SECTIONS"
	echo "  <package>   Package name with mdvl- prefix."
	echo "  <arch>      Rarely necessary: $MDPC_ARCHITECTURES."
	echo
	echo MDPC is released under GPLv3. Check $0 for details.
}

ma_longusage() {
	echo "  new          Create new package or build component."
	echo "  new chroot   Create new chroot."
	echo "  new repo     Create new repository if it not already exists."
	echo "  new dirs     Create basic directory structure."
	echo "  update       Increment version, compile and sync."
	echo "  update_only  Only increment version."
	echo "  compile      Create .deb file (debuild: also compile)."
	echo "  sync         Synchronize to repository."
	echo "  chroot       Enter chroot. admin: be root in it."
	echo "  work         Enter working environment (big terminal REQ)."
	echo "  vm           Start and connect to a test virtual machine."
}

ma_usage() {
	echo "new|update|update_only|compile|sync|work|vm|help"
}

ma_main "$@"
exit $?
